# -*- coding: utf-8 -*-

import sys
from ixiaerror import IxiaError
from ixiangpf import IxiaNgpf
from ixiautil import PartialClass, make_hltapi_fail

class IxiaNgpf(PartialClass, IxiaNgpf):
	def wizard_multivalue_config(self, **kwargs):
		r'''
		#Procedure Header
		 Name:
		    wizard_multivalue_config
		
		 Description:
		    This procedure is used to create multivalue string which can be used to set multivalues in wizard apis
		
		 Synopsis:
		    wizard_multivalue_config
		x       [-start_value      ALPHA
		x                          DEFAULT ]
		x       [-step_value       ALPHA
		x                          DEFAULT ]
		x       [-pattern          CHOICES singleValue
		x                          CHOICES increment
		x                          CHOICES decrement
		x                          DEFAULT singleValue]
		x       [-enable_port_step CHOICES 0 1
		x                          DEFAULT 0]
		x       [-port_step        ALPHA
		x                          DEFAULT ]
		
		 Arguments:
		x   -start_value
		x       The start value of the multivalue pattern
		x   -step_value
		x       The start value of the multivalue pattern
		x   -pattern
		x       The type of multivalue pattern required
		x   -enable_port_step
		x       flag to enable port set in multivalue
		x   -port_step
		x       The port step value of the multivalue pattern
		
		 Return Values:
		    $::SUCCESS | $::FAILURE
		x   key:status             value:$::SUCCESS | $::FAILURE
		    increment decrement or singvalue string
		x   key:multivalue_string  value:increment decrement or singvalue string
		
		 Examples:
		
		 Sample Input:
		
		 Sample Output:
		
		 Notes:
		
		 See Also:
		
		'''
		hlpy_args = locals().copy()
		hlpy_args.update(kwargs)
		del hlpy_args['self']
		del hlpy_args['kwargs']

		not_implemented_params = []
		mandatory_params = []
		file_params = []

		try:
			return self.__execute_command(
				'wizard_multivalue_config', 
				not_implemented_params, mandatory_params, file_params, 
				hlpy_args
			)
		except (IxiaError, ):
			e = sys.exc_info()[1]
			return make_hltapi_fail(e.message)
