##Procedure Header
# Name:
#    ::ixiangpf::emulation_mka_config
#
# Description:
#    This procedure will add Mka(s) to a particular Ixia Interface.
#    The user can then configure, MKA by using the procedure.
#
# Synopsis:
#    ::ixiangpf::emulation_mka_config
#        -mode                          CHOICES create
#                                       CHOICES delete
#                                       CHOICES modify
#                                       CHOICES enable
#                                       CHOICES disable
#        [-port_handle                  REGEXP ^[0-9]+/[0-9]+/[0-9]+$]
#        [-handle                       ANY]
#x       [-active_device                CHOICES 0 1]
#x       [-key_type                     CHOICES psk]
#x       [-randomize_member_identifier  CHOICES 0 1]
#x       [-key_server_priority          ANY]
#x       [-macsec_desired               CHOICES 0 1]
#x       [-macsec_capability            CHOICES macsecnotimplemented
#x                                      CHOICES macsecintegritywithoutconfidentiality
#x                                      CHOICES macsecintegritywithnoconfidentialityoffset
#x                                      CHOICES macsecintegritywithconfidentialityoffset]
#x       [-confidentiality_offset       CHOICES noconfidentiality
#x                                      CHOICES noconfidentialityoffset
#x                                      CHOICES confidentialityoffset30octets
#x                                      CHOICES confidentialityoffset50octets]
#x       [-mka_version                  NUMERIC]
#x       [-mka_hello_time               NUMERIC]
#x       [-mka_life_time                NUMERIC]
#x       [-send_icv_indicator           CHOICES 0 1]
#x       [-delay_protect                CHOICES 0 1]
#x       [-starting_key_number          NUMERIC]
#x       [-starting_distributed_an      NUMERIC]
#x       [-cipher_suite                 CHOICES aes128
#x                                      CHOICES aes256
#x                                      CHOICES aesxpn128
#x                                      CHOICES aesxpn256]
#x       [-rekey_threshold_pn           ANY]
#x       [-rekey_threshold_xpn          ANY]
#x       [-starting_olpn                NUMERIC]
#x       [-olpn_step                    NUMERIC]
#x       [-starting_llpn                NUMERIC]
#x       [-llpn_step                    NUMERIC]
#x       [-rekey_mode                   CHOICES timerBased pNBased]
#x       [-rekey_behaviour              CHOICES dontRekey
#x                                      CHOICES rekeyContinuous
#x                                      CHOICES rekeyFixedCount]
#x       [-periodic_rekey_interval      ANY]
#x       [-periodic_rekey_attempts      NUMERIC]
#x       [-destination_mac_address_type CHOICES nearest_non_tpmr_bridge_group_address
#x                                      CHOICES ede_cc_pep_address
#x                                      CHOICES bridge_group_address
#x                                      CHOICES ede_ss_pep_address
#x                                      CHOICES individual_lan_scope_group_address
#x                                      CHOICES broadcast_address
#x                                      CHOICES custom_address]
#x       [-destination_mac_address      ANY]
#x       [-eapol_ethernet_type          ANY]
#x       [-system_id                    MAC]
#x       [-port_id                      NUMERIC]
#x       [-starting_message_number      NUMERIC]
#x       [-cak_value_128                ANY]
#x       [-cak_value_256                ANY]
#x       [-cak_name                     ANY]
#
# Arguments:
#    -mode
#        This option defines whether to Create/Modify/Delete/Enable/Disable an MKA, along with Tx Channnel List and CAK Cache List under MKA.
#    -port_handle
#        The port on which the MKA configuration will take place.
#    -handle
#        Specifies the parent node/object handle on which the pcep configuration should be configured.
#        In case of create/modify -mode, -handle generally denotes the parent node/object handle .
#        In case of modes – modify/delete/disable/enable -handle denotes the object node handle on which the action needs to be performed.
#x   -active_device
#x       Determines whether an MKA device is active or not. If false, MKA will not be started on the device.
#x   -key_type
#x       Indicates the source of CAK.
#x   -randomize_member_identifier
#x       If this is true, then Ixia MKA devices will generate random member identifiers. Otherwise member identifiers of the form MAC Address-Counter will be generated to aid in debugging.
#x   -key_server_priority
#x       Determines the Key Server Priority.
#x   -macsec_desired
#x       Determines whether MACsec is desired or not. It is advertised in periodic hellos.
#x   -macsec_capability
#x       Indicates MACsec capabilities.
#x   -confidentiality_offset
#x       Determines the confidentiality offset.
#x   -mka_version
#x       Indicates the version of MKA.
#x   -mka_hello_time
#x       Indicates MKA Hello Time.
#x   -mka_life_time
#x       Indicates MKA Life Time of a Peer.
#x   -send_icv_indicator
#x       Send ICV Indicator in MKPDU.
#x   -delay_protect
#x       Delay Protect behavior of the device.
#x   -starting_key_number
#x       Determines the starting key number.
#x   -starting_distributed_an
#x       Determines starting distributed AN.
#x   -cipher_suite
#x       Determines the type of cipher suite.
#x   -rekey_threshold_pn
#x       Determines the PN rekey threshold.
#x   -rekey_threshold_xpn
#x       Determines the XPN rekey threshold.
#x   -starting_olpn
#x       Indicates the starting OLPN.
#x   -olpn_step
#x       Determines the OLPN step.
#x   -starting_llpn
#x       Determines the starting LLPN.
#x   -llpn_step
#x       Determines the LLPN step.
#x   -rekey_mode
#x       Rekey criteria.
#x   -rekey_behaviour
#x       Determines the Rekey behavior.
#x   -periodic_rekey_interval
#x       Time interval between two Rekey triggers.
#x   -periodic_rekey_attempts
#x       Number of times Rekey should be triggered.
#x   -destination_mac_address_type
#x       Determines the Rekey behavior.
#x   -destination_mac_address
#x       Destination MAC Address of the EAPOL PDU.
#x   -eapol_ethernet_type
#x       Determines EAPOL Ethernet Type.
#x   -system_id
#x       System Identifier component of the SCI field with which MACsec traffic is transmitted.
#x   -port_id
#x       Port ID component of SCI with which MACsec traffic is transmitted.
#x   -starting_message_number
#x       Determines the starting message number.
#x   -cak_value_128
#x       Determines the 128 bit CAK value.
#x   -cak_value_256
#x       Determines the 256 bit CAK value.
#x   -cak_name
#x       Denotes CAK name.
#
# Return Values:
#    A list containing the ethernet protocol stack handles that were added by the command (if any).
#x   key:ethernet_handle     value:A list containing the ethernet protocol stack handles that were added by the command (if any).
#    $::SUCCESS | $::FAILURE
#    key:status              value:$::SUCCESS | $::FAILURE
#    When status is $::FAILURE, log shows the detailed information of failure.
#    key:log                 value:When status is $::FAILURE, log shows the detailed information of failure.
#    Handle of MKA configured
#    key:mka_handle          value:Handle of MKA configured
#    Handle of Tx Channels configured
#    key:tx_channels_handle  value:Handle of Tx Channels configured
#    Handle of CAK Caches configured
#    key:cak_cache_handle    value:Handle of CAK Caches configured
#
# Examples:
#
# Sample Input:
#
# Sample Output:
#
# Notes:
#
# See Also:
#

proc ::ixiangpf::emulation_mka_config { args } {

	set notImplementedParams "{}"
	set mandatoryParams "{}"
	set fileParams "{}"
	set flagParams "{}"
	set procName [lindex [info level [info level]] 0]
	::ixia::logHltapiCommand $procName $args
	::ixia::utrackerLog $procName $args
	return [eval runExecuteCommand "emulation_mka_config" $notImplementedParams $mandatoryParams $fileParams $flagParams $args]
}
